/*
 * Copyright
 *   2007 axYus - www.axyus.com
 *   2007 N.Le Corre - nicolas.lecorre@axyus.com
 *
 * This file is part of XEMELIOS.
 *
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package fr.gouv.finances.cp.xemelios.controls.paye;

import java.util.Hashtable;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

import fr.gouv.finances.cp.xemelios.controls.AbstractUnitControl;
import fr.gouv.finances.cp.xemelios.controls.Anomalie;
import fr.gouv.finances.cp.xemelios.controls.IdGenerator;
import fr.gouv.finances.cp.xemelios.controls.Node;

public class PAY12 extends AbstractUnitControl {
	private final static Logger logger = Logger.getLogger(PAY12.class);
    public static final transient String CTRL_ID = PAY12.class.getSimpleName();
	private Vector<Anomalie> anos = new Vector<Anomalie>();
	private String docId = null;

	
	private static final String ON_RETURN_ANOMALIES = "DocumentPaye/DonneesIndiv/PayeIndivMensuel/";
	private static final String CHEMIN_PK_ANNEE = "DocumentPaye/Annee/";
	private static final String CHEMIN_PK_MOIS = "DocumentPaye/Mois/";
	private static final String CHEMIN_PAYE = "DocumentPaye/DonneesIndiv/PayeIndivMensuel/";
	private static final String CHEMIN_PK_MATRICULE = "DocumentPaye/DonneesIndiv/PayeIndivMensuel/Agent/Matricule/";
	private String pk_annee = "", pk_mois = "", pk_matricule = "";
	private String payeNodeId = "";


	private static final String CHEMIN_NOM_AGENT = "DocumentPaye/DonneesIndiv/PayeIndivMensuel/Agent/Nom/";
	private static final String CHEMIN_COMPLEMENT_NOM_AGENT = "DocumentPaye/DonneesIndiv/PayeIndivMensuel/Agent/ComplNom/";
	private static final String REPLACE_PAYE = "##PAYE##";
	private String nom = "", cmpl_nom = "";
	private String nomNodeId = "", cmpl_nomNodeId = "";
	
	public void startDocument() throws SAXException {
		logger.info("In "+CTRL_ID);
	}
	
	public void setDocId(String docID) {
		this.docId = docID;		
	}

	public void startElement(String uri, String localName, String qName, Attributes atts, String xpath) throws SAXException {
		if (xpath.endsWith(CHEMIN_PAYE)) {
			anos = new Vector<Anomalie>();
			payeNodeId = atts.getValue("c");
			pk_matricule = "";
			nom = cmpl_nom = "";
			nomNodeId = cmpl_nomNodeId = "";
		} else if (xpath.endsWith(CHEMIN_PK_ANNEE)) {
			pk_annee = atts.getValue("V");
		} else if (xpath.endsWith(CHEMIN_PK_MOIS)) {
			pk_mois = atts.getValue("V");
		} else if (xpath.endsWith(CHEMIN_PK_MATRICULE)) {
			pk_matricule = atts.getValue("V");
		} else if (xpath.endsWith(CHEMIN_NOM_AGENT)) {
			nom = atts.getValue("V");
			nomNodeId = atts.getValue("ano:node-id");
		} else if (xpath.endsWith(CHEMIN_COMPLEMENT_NOM_AGENT)) {
			cmpl_nom = atts.getValue("V");
			cmpl_nomNodeId = atts.getValue("ano:node-id");
		}
	}

	public Vector<Anomalie> endElement(String uri, String localName, String qName, String xpath) throws SAXException {
		if (xpath.endsWith(ON_RETURN_ANOMALIES)) {
			// ANOMALIE
			if (nom.equals(cmpl_nom)) {
				String msg = getDocumentModel().getControlById(CTRL_ID).getMessage().getHtmlContent();
				msg = msg.replaceAll(REPLACE_PAYE, pk_annee+" / "+pk_mois+" / "+pk_matricule);
				String path="@added:primary-key='"+pk_annee+"-"+pk_mois+"-"+pk_matricule+"'";
				String anoId = IdGenerator.nextId();
				String libelleLien = "";
				Hashtable<String, Object> xslParams = new Hashtable<String, Object>();
				xslParams.put("elementId", "Paye");
				xslParams.put("browser-destination", "internal");
				xslParams.put("anoId", anoId);
				Anomalie ano = new Anomalie(anoId,
											getDocumentModel().getControlById(CTRL_ID).getId(),
											getDocumentModel().getControlById(CTRL_ID).getLibelle(),
											getDocumentModel().getDocumentId(),
											"etatPaye",
											"payeIndivMensuel", // lment importable
											payeNodeId, // id lment importable
											localName,
											libelleLien,
											msg,
											getDocumentModel().getControlById(CTRL_ID).getRegle().getHtmlContent(),
											getDocumentModel().getControlById(CTRL_ID).getSeverity(),
											path, // xpath vers le bordereau sous lequel se trouve l'anomalie
											xslParams); // paramtres supplmentaires pour accder  l'lment en anomalie
				ano.addNode(new Node(nomNodeId));
				ano.addNode(new Node(cmpl_nomNodeId));
				anos.add(ano);
			}
			return anos;
		} else {
			return null;
		}
	}

	public void characters(char[] ch, int start, int length, String xpath)
			throws SAXException {
		// TODO Raccord de mthode auto-gnr

	}

	public void endDocument() throws SAXException {
		// TODO Raccord de mthode auto-gnr

	}

	public void endPrefixMapping(String prefix, String xpath)
			throws SAXException {
		// TODO Raccord de mthode auto-gnr

	}

	public void ignorableWhitespace(char[] ch, int start, int length,
			String xpath) throws SAXException {
		// TODO Raccord de mthode auto-gnr

	}

	public void processingInstruction(String target, String data, String xpath)
			throws SAXException {
		// TODO Raccord de mthode auto-gnr

	}

	public void setDocumentLocator(Locator locator) {
		// TODO Raccord de mthode auto-gnr

	}

	public void setParameters(Hashtable<String, Object> params) {
		// TODO Raccord de mthode auto-gnr

	}

	public void skippedEntity(String name, String xpath) throws SAXException {
		// TODO Raccord de mthode auto-gnr

	}

	public void startPrefixMapping(String prefix, String uri, String xpath)
			throws SAXException {
		// TODO Raccord de mthode auto-gnr

	}

}
