<?xml version="1.0" encoding="ISO-8859-1"?>
<!DOCTYPE xsl:stylesheet [
<!ENTITY nbsp "&#160;">
<!ENTITY egrave "&#232;">
<!ENTITY euro "&#8364;">
<!ENTITY agrave "&#224;">
<!ENTITY eacute "&#233;">
]>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns="http://www.xemelios.org/namespaces#cg-1.0" xmlns:n="http://www.xemelios.org/namespaces#cg-1.0" xmlns:added="http://projets.admisource.gouv.fr/xemelios/namespaces#added" version="2.0">
    <!-- Paramtres de sortie -->
    <xsl:character-map name="accents">
        <xsl:output-character character="" string="&amp;#224;"/>
        <xsl:output-character character="" string="&amp;#233;"/>
        <xsl:output-character character="" string="&amp;#232;"/>
        <xsl:output-character character="" string="&amp;#234;"/>
        <xsl:output-character character="" string="&amp;#235;"/>
        <xsl:output-character character="" string="&amp;#238;"/>
        <xsl:output-character character="" string="&amp;#239;"/>
        <xsl:output-character character="" string="&amp;#244;"/>
        <xsl:output-character character="" string="&amp;#249;"/>
    </xsl:character-map>
    <xsl:output method="xhtml" indent="yes" use-character-maps="accents" encoding="ISO-8859-1"/>
    <xsl:decimal-format name="decformat" decimal-separator="," grouping-separator=" " digit="#" pattern-separator=";" NaN="NaN" minus-sign="-"/>

    <!-- Inclusion des XSL externes -->
    <xsl:include href="./common-xsl/Style.xsl"/>
    <xsl:include href="./common-xsl/Error.xsl"/>
    <xsl:include href="./common-xsl/Navigate.xsl"/>
    <xsl:include href="./common-xsl/Header.xsl"/>
    <xsl:include href="./common-xsl/Number.xsl"/>

    <!-- Paramtres d'entre -->
    <xsl:param name="show.formulaire"/>
    <xsl:param name="browser-destination"/>

    <!-- Paramtres d'entre de la liste de rsultat -->
    <xsl:param name="generated.id">0</xsl:param>

    <!-- Paramtres d'erreur -->
    <xsl:param name="error.message">nomessage</xsl:param>

    <!-- Variables -->
    <xsl:variable name="page-format" select="paysage"/>
    <xsl:variable name="NumPage" select="/n:CompteGestion/n:RenseignementsAnnexesBalance/n:PageRenseignementsAnnexesBalance/n:Pied/@NumPage"/>
    <xsl:variable name="Entete">
        <xsl:copy-of select="/n:CompteGestion/n:Entete"/>
    </xsl:variable>

    <!-- pour eviter les sorties parasites de tags non matches -->
    <xsl:template match="*"/>

    <xsl:template match="/n:CompteGestion">
        <html>
            <head>
                <title>Renseignements annexes &#224; la balance</title>
                <xsl:call-template name="style"/>
            </head>
            <body width="100%">
                <xsl:if test="$error.message!='nomessage'">
                    <xsl:attribute name="onload">javascript:showDialog('<xsl:value-of select="$error.message"/>');</xsl:attribute>
                </xsl:if>
                <xsl:call-template name="Error"><!--xsl:with-param name="error.message" select="$error.message"/--></xsl:call-template>
                <xsl:call-template name="navigate">
                    <xsl:with-param name="docId">compteGestion</xsl:with-param>
                    <xsl:with-param name="etatId">RenseignementsAnnexesBalance</xsl:with-param>
                    <xsl:with-param name="elementId">PageRenseignementsAnnexesBalance</xsl:with-param>
                    <xsl:with-param name="sous.elementId">LigneRenseignementsAnnexesBalance</xsl:with-param>
                    <xsl:with-param name="NumPage" select="$NumPage"/>
                    <xsl:with-param name="Entete" select="$Entete"/>
                    <xsl:with-param name="show.formulaire" select="$show.formulaire"/>
                    <xsl:with-param name="show.formulaire.compte" select="0"/>
                    <xsl:with-param name="show.depense.recette" select="0"/>
                </xsl:call-template>
                <br/>
                <br/>
                <br/>
                <xsl:call-template name="header">
                    <xsl:with-param name="Entete" select="$Entete"/>
                    <xsl:with-param name="Titre">Renseignements annexes &#224; la balance</xsl:with-param>
                </xsl:call-template>
                <br/>
                <br/>
                <br/>
                <br/>
                <xsl:choose>
                    <xsl:when test="n:RenseignementsAnnexesBalance/n:PageRenseignementsAnnexesBalance/@Modele='20'">
                        <xsl:call-template name="mainTable.1">
                            <xsl:with-param name="el" select="/n:CompteGestion/n:RenseignementsAnnexesBalance/n:PageRenseignementsAnnexesBalance"/>
                        </xsl:call-template>
                    </xsl:when>
                    <xsl:when test="n:RenseignementsAnnexesBalance/n:PageRenseignementsAnnexesBalance/@Modele='21'">
                        <xsl:call-template name="mainTable.2">
                            <xsl:with-param name="el" select="/n:CompteGestion/n:RenseignementsAnnexesBalance/n:PageRenseignementsAnnexesBalance"/>
                        </xsl:call-template>
                    </xsl:when>
                </xsl:choose>
                <br/>
                <br/>
                <br/>
                <br/>
                <p>
                    <center>Page - <xsl:value-of select="$NumPage"/></center>
                </p>
            </body>
        </html>
    </xsl:template>

    <xsl:template name="mainTable.1">
        <xsl:param name="el"/>
        <table width="100%" id="main" class="bordered">
            <colgroup>
                <col width="40%"/>
                <col width="10%"/>
                <col width="10%"/>
                <col width="10%"/>
                <col width="10%"/>
                <col width="10%"/>
                <col width="10%"/>

            </colgroup>
            <thead>
                <tr class="titre">
                    <td rowspan="2" class="titre bold center bordered">COMPTES - LIBELLES</td>
                    <td colspan="2" class="titre bold center bordered">RESULTAT A LA CLOTURE DE L&apos;EXERCICE PRECEDENT</td>
                    <td colspan="2" class="titre bold center bordered">RESULTAT DE L&apos;EXERCICE</td>
                    <td colspan="2" class="titre bold center bordered">RESULTAT A LA CLOTURE DE L&apos;EXERCICE</td>
                </tr>
                <tr>
                    <td class="titre2 bold center bordered">DEFICIT</td>
                    <td class="titre2 bold center bordered">EXCEDENT</td>
                    <td class="titre2 bold center bordered">DEFICIT</td>
                    <td class="titre2 bold center bordered">EXCEDENT</td>
                    <td class="titre2 bold center bordered">DEFICIT</td>
                    <td class="titre2 bold center bordered">EXCEDENT</td>
                </tr>
            </thead>
            <tbody>
                <xsl:for-each select="$el/n:LigneRenseignementsAnnexesBalance">
                    <tr>
                        <xsl:choose>
                            <xsl:when test="$generated.id!='0' and @added:generated-id=$generated.id">
                                <xsl:attribute name="id">highlighted</xsl:attribute>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:choose>
                                    <xsl:when test="(position() mod 2) = 0">
                                        <xsl:attribute name="id">colorised1</xsl:attribute>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <xsl:attribute name="id">colorised2</xsl:attribute>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </xsl:otherwise>
                        </xsl:choose>
                        <xsl:choose>
                            <xsl:when test="contains(lower-case(@NumCpte),'total')">
                                <td class=" right bordered bold">
                                    <xsl:value-of select="@NumCpte"/>
                                    <br/>
                                    <xsl:value-of select="@LibelleBudgetAnnexe"/>
                                </td>
                                <td class="montant bordered bold">
                                    <xsl:call-template name="number">
                                        <xsl:with-param name="num" select="@ResultatClotBGPrDeficit"/></xsl:call-template>
                                </td>
                                <td class="montant bordered bold">
                                    <xsl:call-template name="number">
                                        <xsl:with-param name="num" select="@ResultatClotBGExPrecExcedent"/></xsl:call-template>
                                </td>
                                <td class="montant bordered bold">
                                    <xsl:call-template name="number">
                                        <xsl:with-param name="num" select="@ResBGExDeficit"/></xsl:call-template>
                                </td>
                                <td class="montant bordered bold">
                                    <xsl:call-template name="number">
                                        <xsl:with-param name="num" select="@ResBGExExce"/></xsl:call-template>
                                </td>
                                <td class="montant bordered bold">
                                    <xsl:call-template name="number">
                                        <xsl:with-param name="num" select="@ResBGClotExDeficit"/></xsl:call-template>
                                </td>
                                <td class="montant bordered bold">
                                    <xsl:call-template name="number">
                                        <xsl:with-param name="num" select="@ResBGClotExExce"/></xsl:call-template>
                                </td>
                            </xsl:when>
                            <xsl:otherwise>
                                <td class="bordureRight">
                                    <xsl:value-of select="@NumCpte"/>
                                    <br/>
                                    <xsl:value-of select="@LibelleBudgetAnnexe"/>
                                </td>
                                <td class="montant bordureRight">
                                    <xsl:call-template name="number">
                                        <xsl:with-param name="num" select="@ResultatClotBGPrDeficit"/></xsl:call-template>
                                </td>
                                <td class="montant bordureRight">
                                    <xsl:call-template name="number">
                                        <xsl:with-param name="num" select="@ResultatClotBGExPrecExcedent"/></xsl:call-template>
                                </td>
                                <td class="montant bordureRight">
                                    <xsl:call-template name="number">
                                        <xsl:with-param name="num" select="@ResBGExDeficit"/></xsl:call-template>
                                </td>
                                <td class="montant bordureRight">
                                    <xsl:call-template name="number">
                                        <xsl:with-param name="num" select="@ResBGExExce"/></xsl:call-template>
                                </td>
                                <td class="montant bordureRight">
                                    <xsl:call-template name="number">
                                        <xsl:with-param name="num" select="@ResBGClotExDeficit"/></xsl:call-template>
                                </td>
                                <td class="montant bordureRight">
                                    <xsl:call-template name="number">
                                        <xsl:with-param name="num" select="@ResBGClotExExce"/></xsl:call-template>
                                </td>
                            </xsl:otherwise>
                        </xsl:choose>

                    </tr>
                </xsl:for-each>
            </tbody>
        </table>
    </xsl:template>

    <xsl:template name="mainTable.2">
        <xsl:param name="el"/>
        <table width="100%" id="main" class="bordered">
            <colgroup>
                <col width="12%"/>
                <col width="12%"/>
                <col width="12%"/>
                <col width="8%"/>
                <col width="8%"/>
                <col width="8%"/>
                <col width="8%"/>
                <col width="8%"/>
                <col width="8%"/>
                <col width="8%"/>
                <col width="8%"/>

            </colgroup>
            <thead>
                <tr class="titre">
                    <td colspan="2" class="titre bold center bordered">
                        <xsl:value-of select="$el/n:LigneRenseignementsAnnexesBalance/@NumCpteBis"/>
                    </td>
                    <td rowspan="2" class="titre2 bold center bordered">DIFFERENCE BRUTE</td>
                    <td rowspan="2" class="titre2 bold center bordered">
                        <xsl:value-of select="$el/n:LigneRenseignementsAnnexesBalance/@NumCpteTypeCpte3"/>
                    </td>
                    <td rowspan="2" class="titre2 bold center bordered">
                        <xsl:value-of select="$el/n:LigneRenseignementsAnnexesBalance/@NumCpteTypeCpte4"/>
                    </td>
                    <td rowspan="2" class="titre2 bold center bordered">
                        <xsl:value-of select="$el/n:LigneRenseignementsAnnexesBalance/@NumCpteTypeCpte7"/>
                    </td>
                    <td rowspan="2" class="titre2 bold center bordered">
                        <xsl:value-of select="$el/n:LigneRenseignementsAnnexesBalance/@NumCpteTypeCpte5"/>
                    </td>
                    <td rowspan="2" class="titre2 bold center bordered">
                        <xsl:value-of select="$el/n:LigneRenseignementsAnnexesBalance/@NumCpteTypeCpte9"/>
                    </td>
                    <td rowspan="2" class="titre2 bold center bordered">
                        <xsl:value-of select="$el/n:LigneRenseignementsAnnexesBalance/@NumCpteTypeCpte6"/>
                    </td>
                    <td rowspan="2" class="titre2 bold center bordered">
                        <xsl:value-of select="$el/n:LigneRenseignementsAnnexesBalance/@NumCpteTypeCpte8"/>
                    </td>
                    <td rowspan="2" class="titre2 bold center bordered">RESULTAT D&apos;INVESTISSEMENT CUMULE</td>
                </tr>
                <tr>
                    <td class="titre2 bold center bordered">SOLDES DEBITEURS</td>
                    <td class="titre2 bold center bordered">SOLDES CREDITEURS</td>
                </tr>
            </thead>
            <tbody>
                <xsl:for-each select="$el/n:LigneRenseignementsAnnexesBalance">
                    <tr>
                        <xsl:attribute name="id">colorised2</xsl:attribute>

                        <td class="montant bordureRight">
                            <xsl:call-template name="number">
                                        <xsl:with-param name="num" select="@AffectResultSoldeDebiteur"/></xsl:call-template>
                        </td>
                        <td class="montant bordureRight">
                            <xsl:call-template name="number">
                                        <xsl:with-param name="num" select="@AffectResultSoldeCrediteur"/></xsl:call-template>
                        </td>
                        <td class="montant bordureRight">
                            <xsl:call-template name="number">
                                        <xsl:with-param name="num" select="@AffectResultDiffBrute"/></xsl:call-template>
                        </td>
                        <td class="montant bordureRight">
                            <xsl:call-template name="number">
                                        <xsl:with-param name="num" select="@SoldeBSCompte10685"/></xsl:call-template>
                        </td>
                        <td class="montant bordureRight">
                            <xsl:call-template name="number">
                                        <xsl:with-param name="num" select="@SoldeBSCompte10686"/></xsl:call-template>
                        </td>
                        <td class="montant bordureRight">
                            <xsl:call-template name="number">
                                        <xsl:with-param name="num" select="@SoldeBSCompte10687"/></xsl:call-template>
                        </td>
                        <td class="montant bordureRight">
                            <xsl:call-template name="number">
                                        <xsl:with-param name="num" select="@SoldeBSCompte110"/></xsl:call-template>
                        </td>
                        <td class="montant bordureRight">
                            <xsl:call-template name="number">
                                        <xsl:with-param name="num" select="@SoldeBSCompte111"/></xsl:call-template>
                        </td>
                        <td class="montant bordureRight">
                            <xsl:call-template name="number">
                                        <xsl:with-param name="num" select="@SoldeBSCompte119"/></xsl:call-template>
                        </td>
                        <td class="montant bordureRight">
                            <xsl:call-template name="number">
                                        <xsl:with-param name="num" select="@SoldeBSCompte114"/></xsl:call-template>
                        </td>
                        <td class="montant bordureRight">
                            <xsl:call-template name="number">
                                <xsl:with-param name="num" select="@ResultatInvestCumule"/></xsl:call-template>
                        </td>

                    </tr>
                </xsl:for-each>
            </tbody>
        </table>
    </xsl:template>

</xsl:stylesheet>
