/*
 * Copyright
 *   2007 axYus - www.axyus.com
 *   2005 C.Marchand - christophe.marchand@axyus.com
 *
 * This file is part of XEMELIOS.
 *
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package fr.gouv.finances.cp.xemelios.updater.config;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import fr.gouv.finances.cp.utils.xml.marshal.InvalidXmlDefinition;
import fr.gouv.finances.cp.utils.xml.marshal.XmlAttributes;
import fr.gouv.finances.cp.utils.xml.marshal.XmlMarshallable;
import fr.gouv.finances.cp.utils.xml.marshal.XmlOutputter;
import javax.xml.namespace.QName;

public class ReleaseModel implements XmlMarshallable {

    public static final transient String TAG = "release";
    public static final transient QName QN = new QName(TAG);
    private static SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
    private String v;
    private Date date;

    public ReleaseModel(QName tag) {
        super();
    }

    public void addCharacterData(String cData) throws SAXException {
    }

    public void addChild(XmlMarshallable child, QName tagName) throws SAXException {
    }

    public XmlMarshallable getAttributes(XmlAttributes attributes) throws SAXException {
        v = attributes.getValue("v");
        String sTmp = attributes.getValue("date");
        try {
            date = sdf.parse(sTmp);
        } catch (ParseException pEx) {
            throw new SAXException("format for date attribute is YYYY-MM-DD");
        }
        return this;
    }

    public void marshall(XmlOutputter output) {
        output.startTag(TAG);
        output.addAttribute("v", v);
        output.addAttribute("date", sdf.format(date));
    }

    public Date getDate() {
        return date;
    }

    public void setDate(Date date) {
        this.date = date;
    }

    public String getV() {
        return v;
    }

    public void setV(String v) {
        this.v = v;
    }

    public void validate() throws InvalidXmlDefinition {
        if (v == null || v.length() == 0) {
            throw new InvalidXmlDefinition("attribute v is mandatory for " + TAG + " element.");
        }
        if (date == null) {
            throw new InvalidXmlDefinition("attribute date is mandatory for " + TAG + " element.");
        }
    }

    public ReleaseModel clone() {
        ReleaseModel rm = new ReleaseModel(QN);
        rm.v = this.v;
        rm.date = (Date) this.date.clone();
        return rm;
    }

    public XmlMarshallable getChildToModify(String uri, String localName, String qName, Attributes atts) {
        return null;
    }

    public QName getQName() {
        return QN;
    }
}
